#!/usr/bin/env python

############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################

import sys, os, unittest
import socket, urlparse
from ConfigParser import ConfigParser
from ZSI.wstools.WSDLTools import WSDLReader
from ZSI.wstools.TimeoutSocket import TimeoutError
from ZSI.wsdl2python import WriteServiceModule
from pyGridWare.ogsi.security.SecurityHandler import AuthenticationHandler, \
    SignContentsHandler, SignatureHandler

CONFIG_FILE = 'config.txt'
SEC_LOCATION = 'Location'
SEC_PATHS = 'Paths'
SEC_REFERENCES = 'schemaPath'
TEST = 'NotificationFactoryService'
MODULE_DIR = 'stubs'


"""This test file generates the types and service files, then runs
   several tests.
"""

class Holder: 
    """This is just for specifying an element w/o content in
       an instance.
    """
    pass

def createService(url, path, schemaPath):
    """Generate types and services modules if no record of them
       in the moduleDict class variable.
    """
    #Generate Modules
    reader = WSDLReader()
    wsdl = reader.loadFromURL(schemaPath)
    wsm = WriteServiceModule(wsdl, 'pyGridWare.ogsi.Decorator', 'OGSIDecorator')
    wsm.write(False)
    typesModuleName, serviceModuleName = wsm.get_module_names()
    exec('import %s as typesModule' %typesModuleName)
    exec('import %s as serviceModule' %serviceModuleName)

    #Get Locator, grab a port, and set up security.
    kw = {'tracefile':sys.stdout, 'url':path}
    locator = serviceModule.NotificationFactoryServiceLocator()
    port = locator.getNotificationFactory(portAddress=url, **kw)
    auth = AuthenticationHandler()
    auth.setDelegation()
    port.binding.addSecurityHandler(auth)
    port.binding.addSecurityHandler(SignContentsHandler())
    port.binding.addSecurityHandler(SignatureHandler())

    #Create Request message to send off 
    req = serviceModule.CreateServiceInputMessageWrapper()
    req._terminationTime = Holder()
    req._creationParameters = Holder()

    #Send off request, and look at response
    msg = port.createService(req)
    print "RESPONSE: ", msg
    print "\tlocator = ", msg._locator
    print "\treference = ", msg._locator._reference
    print "\tdefinitions = ", msg._locator._reference._definitions
    print "\tcurrentTerminationTime = ", msg._currentTerminationTime
    print "\textensibilityOutput = ", msg._extensibilityOutput


def main():
    cp = ConfigParser()
    cp.read(CONFIG_FILE)
    location = cp.get(SEC_LOCATION, TEST)
    if not location:
        host = socket.gethostbyaddr(socket.gethostname())[2][0]
        port = 8080
        netloc = '%s:%d'%(host,port)
    else:
        host,port = eval(location)
        netloc = '%s:%d'%(host,port)

    path = cp.get(SEC_PATHS, TEST)
    schemaPath = cp.get(SEC_REFERENCES, TEST)
    urlTuple = ('http',netloc,'/%s' %schemaPath,'','','')
    schemaPath = urlparse.urlunparse(urlTuple)

    urlTuple = ('http',netloc,path,'','','')
    url = urlparse.urlunparse(urlTuple)
    createService(url=url, path=path, schemaPath=schemaPath)


if __name__ == '__main__':
    main()
