#!/usr/bin/env python

from __future__ import generators

import ConfigParser
from pyGridWare.ogsi.TCmodified import AnyElement

CONFIG = 'config.txt'
SECTION_URL = 'targetNamespace'
SECTION_MOD = 'typesModules'
SECTION_MESSAGES = 'serviceModules'
SECTION_CALLBACKS = 'callbackModules'
SECTION_CLASS = 'classWrappers'
SECTION_SERVICES = 'StartUpServices'
SECTION_DISPATCH = 'Dispatch'
SECTION_REFERENCES = 'References'

def GetItems(cp, section):
    """python2.2 workaround, 'items' method is new
       to python2.3
    """
    try:
        d2 = cp._ConfigParser__sections[section]
    except KeyError:
        if section != ConfigParser.DEFAULTSECT:
            raise ConfigParser.NoSectionError(section)
        d2 = {}
    d = cp._ConfigParser__defaults.copy()
    d.update(d2)
    if "__name__" in d:
        del d["__name__"]
    return d.items()


class ConfigureError(Exception):
    pass


class ConfigImportHandler(ConfigParser.ConfigParser):
    """ Return a dictionary {uri:(module,class)}
            uri    -- targetNamespace
            module -- Module containing class
            class  -- class name in module

            eg. { 'http://www.gridforum.org/namespaces/2003/03/OGSI': 
                  (<module 'pyGridWare.ogsi.services.NotificationFactoryService_services'>,
                   'www_gridforum_org_namespaces_2003_03_OGSI')
                }
    """
    def __call__(self, addr=None, port=None, config=None):
        idxDict, tDict, mDict, cbDict, cDict, = {},{},{},{},{}
        self.__namespace_types_dict = {}
        #self.__callback_list = []
        self.__message_dict = {}
        self.__ns_message_dict = {}
        self.__services_dict = {}
        self.__post_dict = {}
        self.__references_dict = {}
        
        self.read(config or CONFIG)
        for obj, d in [(self.__getNSDict(SECTION_URL), idxDict), 
                       (self.__getNSDict(SECTION_MOD), tDict),
                       (self.__getNSDict(SECTION_MESSAGES), mDict),
                       (self.__getNSDict(SECTION_CALLBACKS), cbDict),
                       (self.__getNSDict(SECTION_DISPATCH), self.__post_dict),
                       (self.__getNSDict(SECTION_REFERENCES), self.__references_dict),
                       (self.__getNSDict(SECTION_CLASS), cDict)]:
            try:
                while 1: 
                    id,v =  obj.next()
                    d[id] = v
            except StopIteration,ex:
                pass

        #FOR SERVER, SET UP AVAILABLE SERVICES
        obj = self.__getNSDict(SECTION_SERVICES)
        try:
            while 1: 
                id,class_path =  obj.next()
                cname = class_path.split('.')[-1]
                mname = class_path[:class_path.rfind('.')]
                exec('from %s import %s' %(mname, cname))

                post = self.__post_dict.get(id)
                try:
                    reference = self.__references_dict[id]
                except KeyError, ex:
                    raise ConfigureError, 'Bad configure, no reference to %s' %id

                if self.__services_dict.has_key(post):
                    raise ConfigureError, 'Bad configure, duplicate services for dispatch to %s' %post

                self.__services_dict[post] = \
                    eval(cname)(
                        addr=addr,
                        port=port,
                        post=post,
                        reference=reference)

        except StopIteration,ex:
            pass
        except ConfigParser.NoSectionError, ex:
            pass

        # Get Types Module
        # Set AnyElement types for types modules
        for k,v in tDict.items():
            exec('import %s' %v)
            self.__namespace_types_dict[idxDict[k]] = (eval(v),cDict[k])

        # Get Message(Service) Module
        # Set AnyElement types for message modules
        for k,v in mDict.items():
            msg_key = idxDict[k]
            if tDict.has_key(k) and mDict[k] == tDict[k]:
                self.__message_dict[mDict[k]] = self.__namespace_types_dict[msg_key][0]
                self.__ns_message_dict[msg_key] = self.__namespace_types_dict[msg_key][0]
            else:
                exec('import %s' %v)
                key = v
                self.__message_dict[key] = eval(v)
                self.__ns_message_dict[msg_key] = self.__message_dict[key]
        AnyElement.tns_module_class_dict = self.__namespace_types_dict

    def __getNSDict(self, section):
        """python2.2 work around
        """
        if hasattr(self, 'items'):
            for id,value in self.items(section): yield id,value
        else: 
            for id,value in GetItems(self, section): yield id,value

    def getMessageList(self):
        return self.__message_dict.values()

    def getMessageModuleByName(self, module_name):
        return self.__message_dict[module_name]

    def getNamespaceMessagesDict(self):
        return self.__ns_message_dict

    def getNamespaceTypesDict(self):
        """ dictionary of --
                namespace:class instance of types
        """
        return self.__namespace_types_dict
 
    def getTypesList(self):
        return self.__namespace_types_dict.values()

    def getServiceCallBackInstances(self):
        return self.__services_dict.values()

    def getPost_ServiceCallBacks_Dict(self):
        """ dictionary of --
                post:class instance of callbacks
        """
        return self.__services_dict

    def getPost_ServiceCallBacks_Dict(self):
        return self.__services_dict

    def setServiceCallBacksNodeTree(self):
        from pyGridWare.ogsi.services.GridService import Reference
        nt = Reference.node_tree
        for pyobj in self.__services_dict.values():
            nt.setNode(pyobj, pyobj.post)


def main():
    cp = ConfigImportHandler()
    cp('localhost', 8080)

if __name__ == '__main__':
    main()
