############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import base64
from Ft.Xml import XPath
from xml.dom.ext.reader import PyExpat
from xml.ns import DSIG
from xml.dom.ext import c14n
from pyGridWare.ogsi.security.GssSignedInfo import GssSignedInfo

from pyGridWare import getLogger
logger = getLogger(__name__)

class VerifyError(Exception): pass

class XMLSignature:
    def sign(self, context):
        pass

    def verify(self, context):
        pass


class GssXMLSignature(XMLSignature):
    processorNss = {\
        'wsse':'http://schemas.xmlsoap.org/ws/2002/04/secext',
        'wsu':'http://schemas.xmlsoap.org/ws/2002/04/utility',
        'ds':'http://www.w3.org/2000/09/xmldsig#',
        }
    """
    public GssXMLSignature(
        Document doc,
        String BaseURI
    ) throws XMLSecurityException {
        super(doc, BaseURI, SignatureGss.URI);
    }
    """
    def __init__(self, doc):
        """context -- security context
           context_id -- set as KeyName
        """
        self._doc = doc
        self._si = GssSignedInfo(doc=doc)

    def addKeyInfoContextId(contextID):
        return
  
    def getKeyInfoContextId():
        return

    def getDocFromString(self, xmlInstance):
        """xmlInstance -- string
           return DOM Document
        """
        reader = PyExpat.Reader()
        return reader.fromString(xmlInstance)

    def canonicalize(self, dom, **kw):
        """return c14n string
        """
        return c14n.Canonicalize(dom, **kw)

    def digest(self, node):
        """xmlInstance -- string
           return base64 encoded digest value.
        """
        cxml = self.canonicalize(node)
        digest = sha.sha(cxml).digest()
        return base64.encodestring(digest).strip()

    def verify(self, context, contextID):
        if not self._si.verify():
             raise VerifyError, 'digest values failed to match'
        if not self._si.verifySignatureValue(context):
             raise VerifyError, 'Signature failed to verify'
        return True

    def sign(self, context, contextID):
        self._si.generateDigestValues()
        soapMsg = self.canonicalize(self._doc)
        document = self.getDocFromString(soapMsg)
        xContext = XPath.Context.Context(document, processorNss=self.processorNss)

        dExpression = XPath.Compile('//ds:Signature/ds:SignedInfo')
        signedInfoNode = dExpression.evaluate(xContext)[0]
        signedInfoStr = self.canonicalize(signedInfoNode)
        signatureValue = context.getMIC(signedInfoStr)

        signatureValueNode = document.createElementNS(DSIG.BASE, "ds:SignatureValue")
        signatureValueNode.appendChild(document.createTextNode(base64.encodestring(signatureValue)))

        dExpression = XPath.Compile('//ds:Signature')
        signature = dExpression.evaluate(xContext)[0]
        signature.appendChild(signatureValueNode)

        #########################################
        # Add KeyInfo.KeyName
        #########################################
        keyNameNode = document.createElementNS(DSIG.BASE, "ds:KeyName")
        keyNameNode.appendChild(document.createTextNode(contextID))

        xContext = XPath.Context.Context(document, processorNss=self.processorNss)
        dExpression = XPath.Compile('//ds:Signature/ds:KeyInfo')
        keyInfoNode = dExpression.evaluate(xContext)[0]
        keyInfoNode.appendChild(keyNameNode)

        soapMsg = self.canonicalize(document)
        return soapMsg




