#! /usr/bin/env python

############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
#
# Copyright (c) 2001 Zolera Systems.  All rights reserved.
###########################################################################

from ZSI import _copyright, ParsedSoap, SoapWriter, TC, ZSI_SCHEMA_URI, \
    FaultFromFaultMessage, _child_elements, _attrs, FaultException
from ZSI.auth import AUTH
import base64, httplib, cStringIO as StringIO, types, time, urlparse

_b64_encode = base64.encodestring

_AuthHeader = '<BasicAuth xmlns="' + ZSI_SCHEMA_URI + '''">
    <Name>%s</Name><Password>%s</Password>
</BasicAuth>'''

from ZSI.client import Binding
from pyGridWare.ogsi.security.GssSecureConversation import SOAPMessage, SecContext
from pyGridWare import _copyright as _copyrightLBNL
from pyGridWare import getLogger
logger = getLogger(__name__)

_copyright = "%s\n\n%s" %(_copyright, _copyrightLBNL)

class SecurityBinding(Binding):
    '''Object that represents a binding (connection) to a SOAP server.
    Once the binding is created, various ways of sending and
    receiving SOAP messages are available, including a "name overloading"
    style.
    '''

    def __init__(self, nsdict=None, ssl=0, url=None, tracefile=None,
    host='localhost', readerclass=None, port=None, typesmodule=None,
    soapaction='"http://www.zolera.com"', **kw):

        Binding.__init__(self, nsdict, ssl, url, tracefile, host, readerclass, 
            port, typesmodule, soapaction, **kw)
        self._context = SecContext()
        self._handlers = []

    def addSecurityHandler(self, handler):
        self._handlers.append(handler)

    def Send(self, url, opname, obj, nsdict=None, soapaction=None, **kw):
        '''Send a message.  If url is None, use the value from the
        constructor (else error). obj is the object (data) to send.
        Data may be described with a requesttypecode keyword, or a
        requestclass keyword; default is the class's typecode (if
        there is one), else Any.
        '''
        # Get the TC for the obj.
        if kw.has_key('requesttypecode'):
            tc = kw['requesttypecode']
        elif kw.has_key('requestclass'):
            tc = kw['requestclass'].typecode
        elif type(obj) == types.InstanceType:
            tc = getattr(obj.__class__, 'typecode')
            if tc is None: tc = TC.Any(opname, aslist=1)
        else:
            tc = TC.Any(opname, aslist=1)

        # Serialize the object.
        s = StringIO.StringIO()
        d = {}
        if self.ns: d[''] = self.ns
        d.update(nsdict or self.nsdict or {})

        # XXX
        #sw = SoapWriter(s, nsdict=d, header=auth_header)
        sw = SoapWriter(s, nsdict=d)
        if kw.has_key('_args'):
            sw.serialize(kw['_args'], tc)
        else:
            sw.serialize(obj, tc, typed=0)
        sw.close()
        soapdata = s.getvalue()

        # Tracing?
        if self.trace:
            print >>self.trace, "_" * 33, time.ctime(time.time()), "REQUEST:"
            print >>self.trace, soapdata

        # XXX SECURITY REQUEST
        if self._context:
            scheme,netloc,path,params,query,fragment  = urlparse.urlparse(self.url)
            soapMsg = SOAPMessage(message=soapdata,
                          context=self._context,
                          URL='http://%s:%d%s' %(self.host,self.port,path))
            for handler in self._handlers:
                handler.handleRequest(soapMsg)
            soapdata = soapMsg.getMessageString()
            # Tracing?
            if self.trace:
                print >>self.trace, "_" * 33, time.ctime(time.time()), "SIGNED REQUEST:"
                print >>self.trace, soapdata
        elif len(self._handlers):
            raise RuntimeError, 'Must set security context for security handlers'

        # Send the request.
        if not self.ssl:
            self.h = httplib.HTTPConnection(self.host, self.port)
        else:
            self.h = httplib.HTTPSConnection(self.host, self.port,
                        **self.ssl_files)
        self.h.connect()
        self.h.putrequest("POST", url or self.url)
        self.h.putheader("Content-length", "%d" % len(soapdata))
        self.h.putheader("Content-type", 'text/xml; charset=utf-8')
        self.h.putheader("SOAPAction", soapaction or self.soapaction)
        if self.auth_style & AUTH.httpbasic:
            val = _b64_encode(self.auth_user + ':' + self.auth_pass) \
                        .replace("\012", "")
            self.h.putheader('Authorization', 'Basic ' + val)
        for header,value in self.user_headers:
            self.h.putheader(header, value)
        self.h.endheaders()
        self.h.send(soapdata)

        # Clear prior receive state.
        self.data, self.ps = None, None

    def ReceiveSOAP(self, readerclass=None, **kw):
        '''Get back a SOAP message.
        '''
        if self.ps: return self.ps
        if not self.IsSOAP():
            raise TypeError(
                'Response is "%s", not "text/xml"' % self.reply_headers.type)
        if len(self.data) == 0:
            raise TypeError('Received empty response')

        # SECURITY
        if self._context:
            payload = self.data
            scheme,netloc,path,params,query,fragment  = urlparse.urlparse(self.url)
            soapMsg = SOAPMessage(message=payload,
                          context=self._context,
                          URL='http://%s:%s%s' %(self.host,self.port,path))
            for handler in self._handlers:
                if hasattr(handler, 'handleResponse'):
                    handler.handleResponse(soapMsg)
            #SOAPData = soapMsg.getMessageString()

        self.ps = ParsedSoap(self.data,
                        readerclass=readerclass or self.readerclass)
        return self.ps

if __name__ == '__main__': print _copyright
