############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from xml.ns import SOAP, DSIG
from Ft.Xml import XPath
from xml.dom.ext import c14n
from pyGridWare.ogsi.security.ElementProxy import ElementProxy
from pyGridWare.ogsi.security import WSSE, WSU


class SignedInfo(ElementProxy):
    """
        <ds:SignedInfo>
            <ds:CanonicalizationMethod Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"></ds:CanonicalizationMethod>
            <ds:SignatureMethod Algorithm="http://www.globus.org/2002/04/xmlenc#gssapi-sign"></ds:SignatureMethod>
            <ds:Reference URI="#digestSource">
                <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"></ds:DigestMethod>
                <ds:DigestValue>23ipYcsXfRxVglxNj1ogMz2TfaY=</ds:DigestValue>
            </ds:Reference>
        </ds:SignedInfo>
    """
    processorNss = {\
        'soapenv':SOAP.ENV,
        'wsse':WSSE.BASE,
        'wsu':WSU.BASE,
        'ds':DSIG.BASE,
        }
    DigestMethod = "http://www.w3.org/2000/09/xmldsig#sha1"
    CanonicalizationMethod = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315"
    SignatureMethod = None
    CanonicalizationFunctions = {CanonicalizationMethod:c14n.Canonicalize}
    siExp = XPath.Compile('soapenv:Envelope/soapenv:Header/wsse:Security/ds:Signature/ds:SignedInfo')

    def __init__(self, doc):
        self._doc = doc
        self._canMethod = self.CanonicalizationMethod
        self._sigMethod = self.SignatureMethod

    def _canonicalize(self, dom, **kw):
        """return c14n string
        """
        if self.CanonicalizationFunctions.has_key(self.CanonicalizationMethod):
            return self.CanonicalizationFunctions[self.CanonicalizationMethod](dom, **kw)
        raise RuntimeError, 'Unknown canonicalization method %s' % self.CanonicalizationMethod

    def getCanonicalizationMethodURI(self):
        """Returns the Canonicalization method URI  
        """
        return self._canMethod

    def getCanonicalizedOctetStream(self):
        """Method getCanonicalizedOctetStream  
        """
        return 

    def getSignatureMethodURI(self):
        """Returns the Signature method URI  
        """
        return self._sigMethod

    def setCanonicalizationMethodURI(CanonicalizationMethodURI):
        """Sets the Canonicalization method URI  
        """
        return

    def setSignatureMethodURI(SignatureMethodURI):
        """Sets the Signature method URI  
        """
        return

    def toElement(self):
        """Serializes the Reference object to an Element  
        """
        return

    def verify(self):
        """Method verify  
        """
    def verifySignatureValue(self):
        """Method verifySignatureValue
        """
        return

